/*
 * (C) Copyright 2002
 * Richard Jones, rjones@nexus-tech.net
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Boot support
 */
#include <common.h>
#include <command.h>

/* calls to drivers/mmc.c
 */
extern int mmc_init(void);
extern int mmc_read(unsigned int start_sec, 
		unsigned int num_bytes, unsigned char *output_buf);

/* Variable to track if
 * mmc init was called 
 * before block read
 */
int mmcinit=0;

static int do_mmc_read(int argc, char *argv[]) {

	void *buf;
	int ret;
	u32 start_sec, num_bytes;

	if (argc != 3)
		goto usage;
	
	start_sec = simple_strtoul(argv[2], NULL, 16);
	num_bytes = simple_strtoul(argv[3], NULL, 16);

	buf = (unsigned char *) STAGE2_LOADADDR;

 	ret = mmc_read(start_sec, num_bytes, buf);
	if (ret) {
		printf("MMC Read Failed!\n");
		return 1;
	}
	return 0;

usage:
	printf("mmc read <startsector> <numbytes>\n");
	return 1;
}

/* call the main init routine
 * from drivers/mmc.c
 */
static int do_mmc_init(void) {

#ifndef XLP_SIM
	if (mmc_init() == 0) 
#endif
	{
		mmcinit = 1;
		return 0;
	}
	return 1;
}

int do_mmc (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	const char *cmd;

	/* need at least two arguments 
	 */
	if (argc < 2) {
		goto usage;
	}

	cmd = argv[1];

	if (strcmp(cmd, "init") == 0) {
		return do_mmc_init();
	}

	if (!mmcinit) {
		printf("MMC Interface not initialized.\nPlease run `mmc init'\n");
		return 1;
	}

	if (strcmp(cmd, "read") == 0) {
		return do_mmc_read(argc - 1, argv + 1);
	}

usage:
	cmd_usage(cmdtp);
	return 1;
}

U_BOOT_CMD(
		mmc, 4,  1,  do_mmc,
		"MMC Subsystem",
		"mmc init\n"
		"mmc read <startsector> <numbytes>"
		);
