/*
 * Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
 * reserved. Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *	1. 	Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 *
 *	2. 	Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the
 *		distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * --------------------------------#NETL_2#--------------------------------
 */

/* This file has the DRAM programming with
 * added hooks for supporting multiple nodes
 * and channels across dual I2C busses. 
 */

#include <common.h>
#include <asm/arch/cpu.h>

#include "clock.h"
#include "ddr_helper.h"
#include "ddr_spd.h"
#include "ddr_leveling.h"

//#define DDR_LVL_DGB

#ifdef  DDR_LVL_DGB
#define ddr_dbg(fmt,args...)     printf(fmt,##args)
#else
#define ddr_dbg(fmt,args...)
#endif
static struct chan_phy cphy[MAX_NODES][DDR_NUM_CHANNELS];

static const struct ddr_tge_data read_level_data =
{
	{ 0x01010101, 0x01010101,
	  0x00000000, 0x00000000, 
	  0x01010101, 0x01010101,
	  0x00000000, 0x00000000, 
	  0x01010101, 0x01010101,
	  0x00000000, 0x00000000,
	  0x01010101, 0x01010101,
	  0x00000000, 0x00000000},
	{ 0x00010001, 0x00010001} 
} ;

static const struct ddr_tge_data write_level_data =
{
	{ 0x01010101, 0x01010101,
	  0x02020202, 0x02020202, 
	  0x04040404, 0x04040404,
	  0x08080808, 0x08080808, 
	  0x10101010, 0x10101010,
	  0x20202020, 0x20202020,
	  0x40404040, 0x40404040,
	  0x80808080, 0x80808080},
	{ 0x01020408, 0x10204080} 
} ;

static const struct ddr_tge_data write_stress_data =
{
	{ 0x00000000, 0x00000000,
	  0xffffffff, 0xffffffff, 
	  0xaaaaaaaa, 0xaaaaaaaa,
	  0x55555555, 0x55555555, 
	  0xaaaaaaaa, 0xaaaaaaaa,
	  0x55555555, 0x55555555,
	  0xffffffff, 0xffffffff,
	  0x00000000, 0x00000000},
	{ 0x00ffaa55, 0xaa55ff00} 
} ;

 
static const int WR_DQS_M1LAUNCH=0; 
static const int WR_DQS_P0LAUNCH=1; 
static const int WR_DQS_P1LAUNCH=2; 
static const int WR_DQS_P2LAUNCH=3; 

static void edge_fsm_upd(struct edge_state* edge_fsm,
			 u8 pass,
			 u8 dly) ;
static void rdstrb_updstrb(volatile u32* mmio,
			   struct edge_state edge_fsm[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES],	       
			   u32 dly) ;
static void wrstrb_updstrb(volatile u32* mmio,
			   struct edge_state edge_fsm[DDR_NUM_BYTE_LANES],			       
			   u32 dly) ;

static int ddr_rdlvl_done(struct rdlvl_state rdlvl_fsm[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES])
{
	int rank, lane ;
	
	for (rank = 0 ; rank < DDR_NUM_RANKS ; rank++) {
		
		for (lane = 0 ; lane < DDR_NUM_BYTE_LANES ; lane++) {
			
			if (rdlvl_fsm[rank][lane].state != RDLVL_FSM_DONE)
				return 0;
		}
	}

	return 1;
}

static void read_strobe_align(int node,
			      int chan,
			      struct ddr_params* params) 
{
	volatile u32* mmio; 

	int dly ;
	int rank;
	int lane;
	struct byte_phy* bphy;
	struct edge_state edge_fsm[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES] ;
	u8 byte_pass[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES] ;
	u8 any_pass ;
	u8 all_pass ;

	ddr_dbg("Starting read strobe align...\n") ;

	mmio = dmc_io_mmio(node, chan) ;

	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		for (lane = 0; lane < DDR_NUM_BYTE_LANES ; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) {
				edge_fsm[rank][lane].state =  EDGE_FSM_FAIL ;
			} else {
				edge_fsm[rank][lane].state =  EDGE_FSM_DONE ;
			}
		}
	}
	
	for(dly=0; dly<128; dly++) {
		rdstrb_updstrb(mmio,
			       edge_fsm,			       
			       dly);
		
		/* Trigger memory read to capture strobes */
		for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
			ddr_tge_txn(mmio,
				    (rank << 9), /* Address */
				    64,          /* Stride */
				    1,           /* Txn Cnt */
				    1) ;         /* Read */
			ddr_tge_data_check(mmio,
					   rank,
					   params,
					   byte_pass[rank],
					   &any_pass,
					   &all_pass) ;
		}
		/* Check for sampled strobe values */
		for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
			for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
				
				if (edge_fsm[rank][lane].state != EDGE_FSM_DONE) {
					edge_fsm_upd(&edge_fsm[rank][lane],
						     byte_pass[rank][lane],
						     dly) ;
					
					if (edge_fsm[rank][lane].state == EDGE_FSM_DONE) {
						bphy = &(cphy[node][chan].rphy[rank].bphy[0]) ;
						bphy[lane].rx_byte_phase_dll = 
							(edge_fsm[rank][lane].last + edge_fsm[rank][lane].first)/2 ;

						ddr_wr_rx_strb_dly(mmio,
								   rank,
								   lane,
								   bphy[lane].rx_byte_phase_dll) ;
						
					}
				}
			}						
		}
	}
}

static void write_strobe_align(int node,
			       int chan,
			       struct ddr_params* params) 
{
	volatile u32* mmio; 
	int dly ;
	int rank;
	int lane;
	struct byte_phy* bphy;
	struct edge_state edge_fsm[DDR_NUM_BYTE_LANES] ;
	u8 byte_pass[DDR_NUM_BYTE_LANES] ;
	u8 rank_pass[DDR_NUM_BYTE_LANES] ;
	u8 any_pass ;
	u8 all_pass ;
	u8 tx_phase_dll ;

	ddr_dbg("Starting write strobe align...\n") ;

	mmio = dmc_io_mmio(node, chan) ;

	/*Disable auto calibration*/
	ddr_rmw_reg(mmio,DDR_CM1,0x0,0x1,20);
	
	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		for (lane = 0; lane < DDR_NUM_BYTE_LANES ; lane++) {
			if (ddr_rank_vld(params,rank) ) {
				if (ddr_lane_vld(params,rank,lane) ) {
					edge_fsm[lane].state =  EDGE_FSM_FAIL ;
				} else {
					edge_fsm[lane].state =  EDGE_FSM_DONE ;
				}
			}
		}
	}
		
	/*Setup TGE data*/
	ddr_tge_setup_data(mmio,
			   &write_stress_data) ; 

	for(dly=0; dly<128; dly++) {

		wrstrb_updstrb(mmio,
			       edge_fsm,
			       dly) ;

		for (lane = 0; lane < DDR_NUM_BYTE_LANES ; lane++) {
			byte_pass[lane] = 1 ;
		}
				
		/* Trigger memory read to capture strobes */
		for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
			if (ddr_rank_vld(params,rank) ) {
		
				ddr_tge_txn(mmio,
					    (rank << 9), /* Address */
					    64,          /* Stride */
					    1,           /* Txn Cnt */
					    0) ;         /* write */
				
				ddr_tge_txn(mmio,
					    (rank << 9), /* Address */
					    64,          /* Stride */
					    1,           /* Txn Cnt */
					    1) ;         /* Read */
				//ddr_dbg("CHKLO: 0x%08x  CHKHI: 0x%08x  ECC: 0x%08x\n",mmio[DDR_TGE_DMVECTLO],mmio[DDR_TGE_DMVECTHI], mmio[DDR_TGE_CBMVECT] );
				ddr_tge_data_check(mmio,
						   rank,
						   params,
						   rank_pass,
						   &any_pass,
						   &all_pass) ;
				
				for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
					byte_pass[lane] = rank_pass[lane] & byte_pass[lane] ;
				}
			}
		}
		
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			
			if (edge_fsm[lane].state != EDGE_FSM_DONE) {
				
				edge_fsm_upd(&edge_fsm[lane],
					     byte_pass[lane],
					     dly) ;
				
				if (edge_fsm[lane].state == EDGE_FSM_DONE) {
				  
					tx_phase_dll = (edge_fsm[lane].last + edge_fsm[lane].first)/2;
							
					//ddr_dbg("WSTRB LANE %d, EYE START: %d END: %d\n",lane,edge_fsm[lane].first,edge_fsm[lane].last) ;
					for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
						bphy = &(cphy[node][chan].rphy[rank].bphy[0]) ;
						bphy[lane].tx_phase_dll = tx_phase_dll ;				
					}
					ddr_wr_tx_strb_dly(mmio,
							   lane,
							   tx_phase_dll) ;
					
				}
			}
		}										
	}
	/*Enable auto calibration*/
	ddr_rmw_reg(mmio,DDR_CM1,0x1,0x1,20);

}

static int adrcmd_align(int node,
			int chan,
			int sr,
			struct ddr_params* params) 
{
	volatile u32* mmio; 
	struct edge_state edge_fsm ;
	int rank ;
	int dly ;
	int full_pass ;
	u8 byte_pass[DDR_NUM_BYTE_LANES] ;
	u8 any_pass ;
	u8 all_pass ;

	mmio = dmc_io_mmio(node, chan) ;

	edge_fsm.state = EDGE_FSM_FAIL ;
	edge_fsm.first = 0 ;
	edge_fsm.last  = 0 ;
	edge_fsm.cnt   = 0 ;

	/* First sweep single rank load (CS,ODT,CKE) */
	/* Only really sweeping CS/ODT but should apply to others*/
	for (dly = 0 ; dly < 128; dly++) {
	       	
		if (sr)
			ddr_wr_sraddr_strb_dly(mmio,
					       dly) ;
		else 
			ddr_wr_mraddr_strb_dly(mmio,
					       dly) ;
		
	
		/* Force a recalibration to update DLL values */
		ddr_dll_calibrate(mmio) ;

		full_pass = 1;

		/* Reissue init sequence in case mode registers 
		 * were corrupted in prior sweep values
		 */
		ddr_ucs_init_seq(mmio) ;

		/* Trigger memory read to capture strobes */
		for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
			if (ddr_rank_vld(params,rank) ) {
				ddr_tge_txn(mmio,
					    (rank << 9), /* Address */
					    64,          /* Stride */
					    8,           /* Txn Cnt */
					    0) ;         /* write */
				
				ddr_tge_txn(mmio,
					    (rank << 9), /* Address */
					    64,          /* Stride */
					    8,           /* Txn Cnt */
					    1) ;         /* Read */
				
				ddr_tge_data_check(mmio,
						   rank,
						   params,
						   byte_pass,
						   &any_pass,
						   &all_pass) ;

				full_pass &= all_pass;
			}
		}
		if (edge_fsm.state != EDGE_FSM_DONE) {

			if (dly != 127) { 
				edge_fsm_upd(&edge_fsm,
					     full_pass,
					     dly) ;
			} else if (edge_fsm.state == EDGE_FSM_PASS) {
				edge_fsm.state = EDGE_FSM_DONE ;
				edge_fsm.last  = 128 ;
			}
			
			if (edge_fsm.state == EDGE_FSM_DONE) {

				if ( (edge_fsm.last - edge_fsm.first) >= MIN_SRADDR_WIN) {
					
					if (sr) {
						cphy[node][chan].addr_srcmd_dll = (edge_fsm.last + edge_fsm.first)/2 ;
						ddr_wr_sraddr_strb_dly(mmio,
								       cphy[node][chan].addr_srcmd_dll) ;
					} else {
						cphy[node][chan].addr_mrcmd_dll = (edge_fsm.last + edge_fsm.first)/2 ;
						ddr_wr_mraddr_strb_dly(mmio,
								       cphy[node][chan].addr_mrcmd_dll) ;
					}
					return 1 ;
				} else {
					return 0 ;
				}
				
			}
		}
	}
	return 0 ;
}

void ddr_addrcmd_align(int node,
			int chan,
			struct ddr_params* params) 
{
	volatile u32* mmio; 

	int cmd_dly; 

	ddr_dbg("Starting address/cmd align...\n") ;

	mmio = dmc_io_mmio(node, chan) ;

	/* Disable auto calibration
	 */
	ddr_disable_autocal(mmio) ;

	/*Setup TGE data*/
	ddr_tge_setup_data(mmio,
			   &write_stress_data) ; 

	adrcmd_align(node,
		     chan,
		     1,
		     params) ;

	cphy[node][chan].addr_mode = 4; 
	
	for(cmd_dly=3; cmd_dly>=1; cmd_dly--) {
		ddr_wr_ntaddr(mmio,cmd_dly) ;
		ddr_wr_ntcsdly(mmio,cmd_dly);
		if (cmd_dly == 1) {
			/* AL - 1*/
			ddr_wr_mr1al(mmio,0x1) ;
		}
		if (adrcmd_align(node,
				 chan,
				 0,
				 params) == 0) {
			ddr_wr_ntaddr(mmio,cmd_dly + 1) ;
			ddr_wr_ntcsdly(mmio,cmd_dly + 1) ;
			cphy[node][chan].addr_mode = cmd_dly + 1 ;
			break ;
		} 
		cphy[node][chan].addr_mode = cmd_dly + 1 ;
	}
	

	/* Reissue init sequence in case mode registers 
	 * were corrupted in prior sweep values
	 */
	if (cphy[node][chan].addr_mode == 1) {
		/* AL - 1*/
		ddr_wr_mr1al(mmio,0x1) ;
	} else {
		/* AL - 2*/
		ddr_wr_mr1al(mmio,0x2) ;
	}

	ddr_ucs_init_seq(mmio);	

	/* Enable auto calibration
	 */
	ddr_enable_autocal(mmio) ;
}

void ddr_print_linfo(int node,
		      int chan,
		      struct ddr_params* params)
{
	int rank ;
	int lane ;
	
	printf("DDR3 %d.%d:\n",node,chan) ; 
	printf("\tAddress Mode        : %dT\n",(cphy[node][chan].addr_mode) ) ;
	printf("\tAddress Delay       : %d cycle(s)\n",cphy[node][chan].addr_rdly) ;
	printf("\tSingle-Rank load DLL: %d/128 cycle\n",cphy[node][chan].addr_srcmd_dll) ;
	printf("\tMulti-Rank  load DLL: %d/128 cycle\n",cphy[node][chan].addr_mrcmd_dll) ;
	printf("\tWr DQS pre-launch   : %d cycle(s)\n",cphy[node][chan].tx_dqs_prelaunch) ;

	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		if (ddr_rank_vld(params,rank) ) {
			printf("R%d WLV CDLY",rank) ;
		}
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) {
				printf ("\t%d/2",
					cphy[node][chan].rphy[rank].bphy[lane].tx_wl_cdly) ;
			}
		}
		printf ("\n") ;
	}
	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		if (ddr_rank_vld(params,rank) ) {
			printf("R%d WLV FDLY",rank) ;
		}		
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) {
				printf ("\t%d/128",
					cphy[node][chan].rphy[rank].bphy[lane].tx_wl_fdll) ;
			}
		}
		printf ("\n") ;
	}
	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		if (ddr_rank_vld(params,rank) ) {
			printf("R%d WPS DLY",rank) ;
		}
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) {
				printf ("\t%d/128",
					cphy[node][chan].rphy[rank].bphy[lane].tx_phase_dll) ;
			}
		}
		printf ("\n") ;
	}
	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		if (ddr_rank_vld(params,rank) ) {
			printf("R%d RLV CDLY",rank) ;
		}
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) {
				printf ("\t%d/2",
					cphy[node][chan].rphy[rank].bphy[lane].rx_strb_cdly) ;
			}
		}
		printf ("\n") ;
	}
	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		if (ddr_rank_vld(params,rank) ) {
			printf("R%d RLV FDLY",rank) ;
		}
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) {
				printf ("\t%d/128",
					cphy[node][chan].rphy[rank].bphy[lane].rx_strb_fdll) ;
			}
		}
		printf ("\n") ;
	}
	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		if (ddr_rank_vld(params,rank) ) {
			printf("R%d RBPS DLY",rank) ;
		}
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) {
				printf ("\t%d/128",
					cphy[node][chan].rphy[rank].bphy[lane].rx_byte_phase_dll) ;
			}
		}
		printf ("\n") ;
	}
}

void ddr_write_level(int node,
		      int chan,
		      struct ddr_params* params)
{
	volatile u32* mmio;
	int rank;
	int lane;
	int i;
	int no_rank_pass;
	struct byte_phy* bphy;
	u8 byte_pass[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES] ;
	u8 lane_done[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES] ;
	u8 any_pass[DDR_NUM_RANKS] ;
	u8 all_pass[DDR_NUM_RANKS] ;

	ddr_dbg("Starting write leveling...\n") ;

	mmio = dmc_io_mmio(node, chan) ;

	/* Disable auto calibration
	 * Disable auto refresh
	 * Disable dynamic ODT
	 */
	ddr_disable_autocal(mmio) ;
	ddr_disable_autoref(mmio) ;
	ddr_disable_dynodt_mode(mmio) ;

	/*Start hardware based write leveling*/
	ddr_do_wlevel(mmio,params->rank_vec) ;
	
	/* Enable auto refresh
	 * Enable dynamic ODT
	 */
	ddr_enable_autoref(mmio) ;
	ddr_enable_dynodt_mode(mmio) ;

	/*Setup TGE data*/
	ddr_tge_setup_data(mmio,
			   &write_level_data) ; 

	/* Store computed write leveling values */
	for (rank = 0 ; rank < DDR_NUM_RANKS ; rank++) {		
		for (lane = 0 ; lane < DDR_NUM_BYTE_LANES ; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) {	
				bphy = &(cphy[node][chan].rphy[rank].bphy[0]) ;
				
				bphy[lane].tx_wl_cdly      = ddr_rd_txwl_cdly(mmio,rank,lane)  ;
				bphy[lane].tx_wl_fdll      = ddr_rd_txwl_fdly(mmio,rank,lane) & 0x7f ;

/*
				if ( (rank%2) == 0) {
					bphy[lane].tx_wl_cdly      = ddr_rd_txwl_cdly(mmio,rank,lane)  ;
					bphy[lane].tx_wl_fdll      = ddr_rd_txwl_fdly(mmio,rank,lane) & 0x7f ;
				} else {
					bphy[lane].tx_wl_cdly      = cphy[node][chan].rphy[rank-1].bphy[lane].tx_wl_cdly ;
					bphy[lane].tx_wl_fdll      = cphy[node][chan].rphy[rank-1].bphy[lane].tx_wl_fdll ;
					ddr_wr_txwl_cdly(mmio,
							 rank,
							 lane,
							 bphy[lane].tx_wl_cdly) ;
					ddr_wr_txwl_fdly(mmio,
							 rank,
							 lane,
							 bphy[lane].tx_wl_fdll) ;
				}
*/
				//ddr_dbg ("WDLY r%dl%d cdly: %d fdly:%d\n",rank,lane,bphy[lane].tx_wl_cdly,bphy[lane].tx_wl_fdll);
				lane_done[rank][lane] = 0 ;
			} else {
				lane_done[rank][lane] = 1  ;
			}
		}
		
	}

	ddr_dll_calibrate(mmio) ;

	for (i = 0 ; i < 4; i++) {
		if (i == WR_DQS_M1LAUNCH) {
			ddr_wr_dqs_prelaunch(mmio,1) ;
		} else if ( (i == WR_DQS_P1LAUNCH) ||
			    (i == WR_DQS_P2LAUNCH) ) {
			
			for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {		
				for (lane = 0; lane < DDR_NUM_BYTE_LANES ; lane++) {
					if (lane_done[rank][lane] == 0) {
						bphy = &(cphy[node][chan].rphy[rank].bphy[0]) ;			
						bphy[lane].tx_wl_cdly+=2 ;
						ddr_wr_txwl_cdly(mmio,
								 rank,
								 lane,
								 bphy[lane].tx_wl_cdly) ;	
					}
				}
			}
		}
		ddr_ucs_init_seq(mmio) ;
		for (rank = 0 ;rank < DDR_NUM_RANKS ;rank++) {
			
			ddr_tge_txn(mmio,
				    (rank << 9), /* Address */
				    64,          /* Stride */
				    8,           /* Txn Cnt */
				    0) ;         /* write */

			ddr_tge_txn(mmio,
				    (rank << 9), /* Address */
				    64,          /* Stride */
				    8,           /* Txn Cnt */
				    1) ;         /* Read */
			
			//ddr_dbg("CHKLO: 0x%08x  CHKHI: 0x%08x  ECC: 0x%08x\n",mmio[DDR_TGE_DMVECTLO],mmio[DDR_TGE_DMVECTHI], mmio[DDR_TGE_CBMVECT] );
			ddr_tge_data_check(mmio,
					   rank,
					   params,
					   byte_pass[rank],
					   &any_pass[rank],
					   &all_pass[rank]) ;        
			
		}
		no_rank_pass = 1 ;
		
		for (rank = 0 ;rank < DDR_NUM_RANKS ;rank++) {
			for (lane = 0; lane < DDR_NUM_BYTE_LANES ; lane++) {
				if (lane_done[rank][lane] == 0) {
					if (byte_pass[rank][lane]) {
						//ddr_dbg("loop: %d rank: %d lane: %d pass\n",i,rank,lane) ;
						bphy = &(cphy[node][chan].rphy[rank].bphy[0]) ;
						lane_done[rank][lane] = 1;
					}
				}
			}

			if (ddr_rank_vld(params,rank) && 
			    any_pass[rank]) {
				no_rank_pass = 0;	
			}
		}
		if (no_rank_pass) {
 			//ddr_dbg("i:%d no rank pass\n",i) ;
	        }
		if (i == WR_DQS_M1LAUNCH) {
			if (no_rank_pass) {
				cphy[node][chan].tx_dqs_prelaunch = 0 ;
				ddr_wr_dqs_prelaunch(mmio,0) ;
			} else {
				cphy[node][chan].tx_dqs_prelaunch = 1 ;
			} 	
		}
	}
	for (rank = 0 ;rank < DDR_NUM_RANKS ;rank++) {
		for (lane = 0; lane < DDR_NUM_BYTE_LANES ; lane++) {
			if (lane_done[rank][lane] != 1) {
				printf("DDR SPD ERROR: Node %d Channel %d write leveling failed\n",node,chan) ;
				return  ;
			}
		}
	}

	write_strobe_align(node,chan,params) ;

	/* Enable auto calibration
	 */
	ddr_enable_autocal(mmio) ;
}

void ddr_adj_turnaround(int node,
			int chan,
			struct ddr_params* params)
{
	volatile u32* mmio;
	struct byte_phy* bphy;
	int rank ;
	int lane ;
	int rdlvl_dly, wrlvl_dly;
	int txmc[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES] ;
	int txmd[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES] ;
	int txmc_slowest ;
	int txmc_fastest ;	
	int txmd_slowest ;
	int cfactor ;
	int dfactor ;

	mmio = dmc_io_mmio(node, chan) ;

	for (rank = 0 ;rank < DDR_NUM_RANKS ;rank++) {
		for (lane = 0; lane < DDR_NUM_BYTE_LANES ; lane++) {	
			if (ddr_lane_vld(params,rank,lane) ) {				
				bphy = &(cphy[node][chan].rphy[rank].bphy[0]) ;
				rdlvl_dly = ( ( (int) bphy[lane].rx_strb_cdly << 6) +
					      ( (int) bphy[lane].rx_strb_fdll) ) ;
				wrlvl_dly = ( ( (int) bphy[lane].tx_wl_cdly << 6) +
					      ( (int) bphy[lane].tx_wl_fdll) ) ;
				
				wrlvl_dly = wrlvl_dly - (cphy[node][chan].tx_dqs_prelaunch << 7) ;
			     
				txmc[rank][lane] = (rdlvl_dly + wrlvl_dly)/2 ;
				txmd[rank][lane] = rdlvl_dly - txmc[rank][lane] ;
			}
		}
	}
	txmc_slowest  = -2048  ;
	txmc_fastest  = 0xffff ;
	txmd_slowest  = -2048  ;

	for (rank = 0 ;rank < DDR_NUM_RANKS ;rank++) {
		for (lane = 0; lane < DDR_NUM_BYTE_LANES ; lane++) {	
			if (ddr_lane_vld(params,rank,lane) ) {	
				if (txmc[rank][lane] < txmc_fastest) {
					txmc_fastest = txmc[rank][lane];
				}
				if (txmc[rank][lane] > txmc_slowest) {
					txmc_slowest = txmc[rank][lane];
				}
				if (txmd[rank][lane] > txmd_slowest) {
					txmd_slowest = txmd[rank][lane];
				}
				
			}
		}
	}
	
	
	cfactor = txmc_slowest - txmc_fastest ;
	dfactor = 2 * txmd_slowest ;

	/* Adjust to cycles - round up*/
	if (cfactor < 0) {
		cfactor = 0;
	} else {
		if ( (cfactor & 0x7f) > 0) {
			cfactor = (cfactor >> 7) + 1;
		} else {
			cfactor = (cfactor >> 7) ;
		}
	}
	if (dfactor < 0) {
		dfactor = 0;
	} else {
		if ( (dfactor & 0x7f) > 0) {
			dfactor = (dfactor >> 7) + 1;
		} else {
			dfactor = (dfactor >> 7) ;
		}
	}
	dfactor++ ;
       
	//ddr_dbg ("cfactor: %d dfactor: %d\n",cfactor,dfactor) ;

	ddr_wr_rdrd_dfrk_ta(mmio,ddr_rd_rdrd_dfrk_ta(mmio) + cfactor ) ;
	ddr_wr_rdwr_smrk_ta(mmio,ddr_rd_rdwr_smrk_ta(mmio) + dfactor) ;
	ddr_wr_rdwr_dfrk_ta(mmio,ddr_rd_rdwr_dfrk_ta(mmio) + (cfactor+dfactor) ) ;
	ddr_wr_wrrd_dfrk_ta(mmio,ddr_rd_wrrd_dfrk_ta(mmio) + cfactor) ;
	ddr_wr_wrwr_dfrk_ta(mmio,ddr_rd_wrwr_dfrk_ta(mmio) + cfactor) ;
	ddr_wr_wrrd_smrk_ta(mmio,ddr_rd_wrrd_smrk_ta(mmio) + 4) ;

	//ddr_dbg("RDRD DF RANK: 0x%08x\n",ddr_rd_rdrd_dfrk_ta(mmio) ) ;
	//ddr_dbg("RDWR SM RANK: 0x%08x\n",ddr_rd_rdwr_dfrk_ta(mmio) ) ;
	//ddr_dbg("RDWR DF RANK: 0x%08x\n",ddr_rd_rdwr_dfrk_ta(mmio) ) ;
	//ddr_dbg("WRRD DF RANK: 0x%08x\n",ddr_rd_wrrd_dfrk_ta(mmio) ) ;
	//ddr_dbg("WRWR DF RANK: 0x%08x\n",ddr_rd_wrwr_dfrk_ta(mmio) ) ;
	//ddr_dbg("WRRD SM RANK: 0x%08x\n",ddr_rd_wrrd_smrk_ta(mmio) ) ;

}

static void edge_fsm_upd(struct edge_state* edge_fsm,
			 u8 pass,
			 u8 dly) 
{
	switch (edge_fsm->state) {
		
	case EDGE_FSM_FAIL:
		if (pass) {
			edge_fsm->state = EDGE_FSM_PASS ;
			edge_fsm->cnt   = 0 ;
			edge_fsm->first = dly ;
		}
		break ;
	case EDGE_FSM_PASS:
		if (pass) {
			edge_fsm->cnt++ ;
		} else {			
			if (edge_fsm->cnt > EDGE_MIN_CNT) {
				edge_fsm->state = EDGE_FSM_DONE ;
				edge_fsm->last  = dly ;
			} else {
				edge_fsm->state = EDGE_FSM_FAIL ;
			}
		}
		break;
	case EDGE_FSM_DONE:
		break;
	default:
		break;
	
	}
}
			  
static void rdlvl_fsm_upd(struct rdlvl_state* rdlvl_fsm,
			  u8 addr_rdly,
			  u8 cdly,
			  u8 fdly,
			  u32 strb_smpl)
{		
	switch (rdlvl_fsm->state) {
		
	case RDLVL_FSM_PREAMBLE:
		if (strb_smpl == 0) {
			rdlvl_fsm->smpl_cnt++ ;
		} else {
			if (rdlvl_fsm->smpl_cnt >= RDLVL_MIN_EDGE_CNT) {
				rdlvl_fsm->state = RDLVL_FSM_EDGE1 ;
				//ddr_dbg("RDLVL_FSM_PREAMBLE: cdly %d lane %d smpl_cnt %d\n",cdly,rdlvl_fsm->lane,rdlvl_fsm->smpl_cnt) ;
				/* Subtract one from cdly */
				/* to center in middle of preamble */
				rdlvl_fsm->cdly      = cdly - 1;
				rdlvl_fsm->fdly      = fdly ;
				rdlvl_fsm->addr_rdly = addr_rdly ;
			}
			rdlvl_fsm->smpl_cnt = 0;
		}
		break ;

	case RDLVL_FSM_EDGE1:
		if (rdlvl_fsm->smpl_cnt < RDLVL_UNDEF_REGION_CNT) {
			rdlvl_fsm->smpl_cnt++ ;
		} else { 
			if (strb_smpl == 1) {
				if (rdlvl_fsm->smpl_cnt > RDLVL_MAX_EDGE_CNT) {
					rdlvl_fsm->state = RDLVL_FSM_PREAMBLE;	
					rdlvl_fsm->smpl_cnt = 0;
				} else {
					rdlvl_fsm->smpl_cnt++ ;
				}
			} else {
				if  ( (rdlvl_fsm->smpl_cnt - 
				       RDLVL_UNDEF_REGION_CNT) > RDLVL_MIN_EDGE_CNT) {
					rdlvl_fsm->state = RDLVL_FSM_EDGE2 ;
					//ddr_dbg("RDLVL_FSM_EDGE1:lane %d  smpl_cnt %d\n",rdlvl_fsm->lane,rdlvl_fsm->smpl_cnt) ;
				} else {
					rdlvl_fsm->state  = RDLVL_FSM_PREAMBLE;
				}
				rdlvl_fsm->smpl_cnt = 0;				
			}
		}
		break;
	case RDLVL_FSM_EDGE2:
		if (rdlvl_fsm->smpl_cnt < RDLVL_UNDEF_REGION_CNT) {
			rdlvl_fsm->smpl_cnt++ ;
		} else { 
			if (strb_smpl == 0) {
				if (rdlvl_fsm->smpl_cnt > RDLVL_MAX_EDGE_CNT) {
					rdlvl_fsm->state  = RDLVL_FSM_PREAMBLE;	
					rdlvl_fsm->smpl_cnt = 0;
				} else {
					rdlvl_fsm->smpl_cnt++ ;
				}					
			} else {
				if  ( (rdlvl_fsm->smpl_cnt - 
				       RDLVL_UNDEF_REGION_CNT) > RDLVL_MIN_EDGE_CNT) {
					rdlvl_fsm->state = RDLVL_FSM_DONE ;
					//ddr_dbg("RDLVL_FSM_DONE: lane %d smpl_cnt %d\n",rdlvl_fsm->lane,rdlvl_fsm->smpl_cnt) ;
				} else {
					rdlvl_fsm->state  = RDLVL_FSM_PREAMBLE;
				}	
				rdlvl_fsm->smpl_cnt = 0;
			}
		}
		break ;
	default:
		break;
	}
}

static void rdlvl_updstrb(volatile u32* mmio,
			  struct rdlvl_state rdlvl_fsm[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES],			       
			  u32 cdly,
			  u32 fdly)
{
	int rank ;
	int lane ;
	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {				
		for (lane = 0; lane < DDR_NUM_BYTE_LANES ; lane++) {					
			if (rdlvl_fsm[rank][lane].state != RDLVL_FSM_DONE) {
				ddr_wr_rx_strben_dly(mmio,
						     rank,
						     lane,
						     cdly,
						     fdly) ;	
			}
		}
	}	
	/* Force a recalibration to update DLL values */
	ddr_dll_calibrate(mmio) ;
}
static void rdstrb_updstrb(volatile u32* mmio,
			   struct edge_state edge_fsm[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES],			       
			   u32 dly)
{
	int rank ;
	int lane ;
	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {				
		for (lane = 0; lane < DDR_NUM_BYTE_LANES ; lane++) {					
			if (edge_fsm[rank][lane].state != EDGE_FSM_DONE) {
				ddr_wr_rx_strb_dly(mmio,
						   rank,
						   lane,
						   dly) ;	
			}
		}
	}	
	/* Force a recalibration to update DLL values */
	ddr_dll_calibrate(mmio) ;
}
static void wrstrb_updstrb(volatile u32* mmio,
			   struct edge_state edge_fsm[DDR_NUM_BYTE_LANES],			       
			   u32 dly)
{
	int lane ;
	for (lane = 0; lane < DDR_NUM_BYTE_LANES ; lane++) {					
		if (edge_fsm[lane].state != EDGE_FSM_DONE) {
			ddr_wr_tx_strb_dly(mmio,
					   lane,
					   dly) ;
		}
	}
	/* Force a recalibration to update DLL values */
	ddr_dll_calibrate(mmio) ;
}

static void rdlvl_pstproc(volatile u32* mmio,
			  struct ddr_params* params,
			  int node,
			  int chan,
			  struct rdlvl_state rdlvl_fsm[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES])			       
			       
{
	int rank ;
	int lane ;
	int addr_rdly ;
	int rd_vld_dly ;
	int dly ;
	int cdly ;
	struct byte_phy* bphy ;

	addr_rdly = 0xff ;
	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) { 
				if (rdlvl_fsm[rank][lane].addr_rdly < addr_rdly) {
					addr_rdly = rdlvl_fsm[rank][lane].addr_rdly ;	
				}
			}
		}
	}
	
	ddr_wr_adddly_rl(mmio,addr_rdly) ;
	cphy[node][chan].addr_rdly = addr_rdly ;

	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		bphy = cphy[node][chan].rphy[rank].bphy ;
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) { 
				rdlvl_fsm[rank][lane].cdly += ( (rdlvl_fsm[rank][lane].addr_rdly - addr_rdly) * 2);
				bphy[lane].rx_strb_cdly = rdlvl_fsm[rank][lane].cdly ;
				bphy[lane].rx_strb_fdll = rdlvl_fsm[rank][lane].fdly ;	
				
				ddr_wr_rx_strben_dly(mmio,
						     rank,
						     lane,
						     bphy[lane].rx_strb_cdly,
						     bphy[lane].rx_strb_fdll) ;
				
				if (bphy[lane].rx_strb_cdly < 1) {
					cdly = 0 ;
				} else {
					if (bphy[lane].rx_strb_fdll > 0) {
						if (bphy[lane].rx_strb_cdly == 1) {
							cdly = 0 ;	
						} else {
							cdly = bphy[lane].rx_strb_cdly - 2 ;	
						}
					} else {
						cdly = bphy[lane].rx_strb_cdly - 1 ;
					}
				}
				ddr_wr_rx_cten_dly(mmio,
						   rank,
						   lane,
						   cdly) ;	
			}		
		}
	}

	ddr_dll_calibrate(mmio) ;

	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		rd_vld_dly = 0 ;
		
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) { 
				dly = ( (rdlvl_fsm[rank][lane].cdly/2) +
					(rdlvl_fsm[rank][lane].cdly%2) + 
					( (rdlvl_fsm[rank][lane].fdly > 0) ? 1:0) + 
					2) ;
				
				if (dly > rd_vld_dly) {
					rd_vld_dly = dly ;
				}
			}
		}
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			ddr_wr_rx_datavld_dly(mmio,
					      rank,
					      lane,
					      rd_vld_dly) ;
		}
		
	}		
}

/* Read leveling */
int ddr_read_level(int node,
		    int chan,
		    struct ddr_params* params)
{
	volatile u32* mmio;	
	u8 level_done ;
	u8 any_lane_done ;
	u8 no_lane_done ;
	u8 cdly,fdly ;
	int rank,lane ;
	u32 strb_smpl ;
	struct rdlvl_state rdlvl_fsm[DDR_NUM_RANKS][DDR_NUM_BYTE_LANES] ;
	u8 addr_rdly ;

	level_done   = 0 ;
	no_lane_done = 1 ;
	addr_rdly    = 0;
	cdly         = 0 ;
	any_lane_done = 0 ;
	
  	mmio = dmc_io_mmio(node, chan) ;

	/* Force DQ & DQS termination
	 * Disable auto calibration
	 * Disable auto refresh
	 * Enter MR3.MPR mode
	 */
	ddr_force_term(mmio) ;
	ddr_disable_autocal(mmio) ;
	ddr_disable_autoref(mmio) ;
	ddr_enter_mpr_mode(mmio);
	
	ddr_wr_adddly_rl(mmio,0) ;
	
	/* Setup data for transaction generator */
	ddr_tge_setup_data(mmio,
			   &read_level_data) ; 

	for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
		for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
			if (ddr_lane_vld(params,rank,lane) ) {
				rdlvl_fsm[rank][lane].state    = RDLVL_FSM_PREAMBLE ;
				rdlvl_fsm[rank][lane].smpl_cnt = 0 ;
				rdlvl_fsm[rank][lane].lane     = lane ;
			} else {
				rdlvl_fsm[rank][lane].state    = RDLVL_FSM_DONE ;
				rdlvl_fsm[rank][lane].lane     = lane ;
			}
		}
	}

	ddr_dbg("Starting read leveling loop....\n") ;

	while (!level_done && 
	       (addr_rdly < 4) &&
	       (cdly < 8) ) {
		
		/* If the coarse delay is 4, increase the address read latency by 1 cycle */
		/* This should only be done if no byte lanes in the channel have completed leveling*/
		if ( (cdly == 4) && !any_lane_done) {
			addr_rdly++ ;
			ddr_wr_adddly_rl(mmio,addr_rdly) ;
			cdly = cdly - 2 ;
		}

		fdly = 0 ;

		while (!level_done &&
		       (fdly < 64) ) {

			rdlvl_updstrb(mmio,
				      rdlvl_fsm,
				      cdly,
				      fdly) ;
		
			/* Check for sampled strobe values */
			for (rank = 0 ; rank < DDR_NUM_RANKS; rank++) {
				if (!ddr_rank_vld(params,rank) ) {
					continue ;
				}

				/* Trigger memory read to capture strobes */
				ddr_tge_txn(mmio,
					    (rank << 9), /* Address */
					    64,          /* Stride */
					    1,           /* Txn Cnt */
					    1) ;         /* Read */

				for (lane = 0; lane < DDR_NUM_BYTE_LANES; lane++) {
					
					if (rdlvl_fsm[rank][lane].state != RDLVL_FSM_DONE) {

						strb_smpl = ddr_rd_strb_sample(mmio,lane) & 0x1;
						 			
						rdlvl_fsm_upd(&rdlvl_fsm[rank][lane],
							      addr_rdly,
							      cdly,
							      fdly,
							      strb_smpl) ;
						if (rdlvl_fsm[rank][lane].state == RDLVL_FSM_DONE) {
							any_lane_done = 1 ; 
							level_done    = ddr_rdlvl_done(rdlvl_fsm) ; 
						}
					}
				}
			}
			fdly = fdly+RDLVL_FDLY_STRIDE ;
		}
		cdly++ ;
	}
	if (level_done == 0) {
		printf("DDR SPD ERROR: Node %d Channel %d Read leveling failed\n",node,chan) ;
		return -1 ;
	}
	ddr_dbg("Starting read leveling post process....\n") ;

	rdlvl_pstproc(mmio,
		      params,
		      node,
		      chan,
		      rdlvl_fsm) ;

	read_strobe_align(node,chan,params) ;

	/* Unforce DQ & DQS termination
	 * Enable auto calibration
	 * Enable auto refresh
	 * Leave MR3.MPR mode
	 */
	ddr_unforce_term(mmio) ;
	ddr_enable_autocal(mmio) ;
	ddr_leave_mpr_mode(mmio);
	ddr_enable_autoref(mmio) ;
	
	return 0;
}
