/*
 * Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
 * reserved. Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *	1. 	Redistributions of source code must retain the above copyright
 *		notice, this list of conditions and the following disclaimer.
 *
 *	2. 	Redistributions in binary form must reproduce the above copyright
 *		notice, this list of conditions and the following disclaimer in
 *		the documentation and/or other materials provided with the
 *		distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * --------------------------------#NETL_2#--------------------------------
 */

/* This file has the DRAM programming with
 * added hooks for supporting multiple nodes
 * and channels across dual I2C busses. 
 */

#include <common.h>
#include <asm/arch/cpu.h>

static int link_up(volatile u32* mmio)
{
	u32 rxsync ;
	int i ;
	/* Poll for 50ms*/
	for (i=0;i<5000;i++) {
		rxsync = mmio[ICI_LA_RX_SYNC] ;
		if (ICI_RX_LANE_SYNC(rxsync) &&
		    ICI_RX_LANE_ALIGN(rxsync) ) {
			return 1 ;
		}
		/* wait 10us*/
		udelay(10);
	}
	return 0;
}
static int check_probe(volatile u32* mmio)
{
	int i ;
	/* Poll for 100us*/
	for (i=0;i<10;i++) {
		if ( (mmio[ICI_CREDLD] & ICI_CREDLD_PRBACK) ) {
			return 1;
		}
		udelay(10);
	}
	printf("Failed to recieve probe ack 0x%08x\n",mmio[ICI_CREDLD]) ;
	return 0;
}

static int advertise_node(volatile u32* mmio)
{
	int i ;
	mmio[ICI_CREDLD] = (mmio[ICI_CREDLD] & ~(ICI_CREDLD_SNDPRB) ) | ICI_CREDLD_SNDNODE;

	/* Poll for 5us*/
	for (i=0;i<5;i++) {
		if ( (mmio[ICI_CREDLD] & ICI_CREDLD_NODERCV) == 0) {
			mmio[ICI_CREDLD] = (mmio[ICI_CREDLD] & ~(ICI_CREDLD_SNDNODE) ) ;
			return 1;
		}
		udelay(1);
	}
	printf("Failed to recieve node ack 0x%08x\n",mmio[ICI_CREDLD]) ;
	return 0;
}
static int link_disable(volatile u32* mmio) 
{
	/* Disable ICI link and put PLL's in reset */
	mmio[ICI_CTL] = 0x0 ;
}
static int enumerate(int node)
{
	volatile u32* mmio_bridge  ;
	volatile u32* mmio ;	
	int peerid = 1;
	int i;

	for (i=0;i<ICI_NUM_LINKS;i++) {

		mmio      = ici_io_mmio(node,i) ;
		if (*(mmio - (HDR_OFFSET>>2) ) != 
		    ( (ICI_DEVICE_ID<<16) | NETL_VENDOR_ID) ) {
			continue ;
		}
		if (link_up(mmio) ) {
			mmio[ICI_CREDLD] = (ICI_CREDLD_PEERID(peerid) |
					    ICI_SFEN |
					    ICI_CDRRSTEN |
					    ICI_CDRLARXRSTEN |
					    ICI_CREDLD_SNDPRB) ;

			if (!check_probe(mmio)) {
				link_disable(mmio) ;
				continue ;
			}

			if (!advertise_node(mmio)) {
				link_disable(mmio) ;
				continue ;
			}
			//printf("ICI LINK%d UP: 0x%08x\n",i,mmio[ICI_CREDLD]) ;			

			if (peerid == 1) {
				mmio_bridge = cpu_io_mmio(node,BRIDGE) ;
				mmio_bridge[NBU_MODE_REG] = mmio_bridge[NBU_MODE_REG] | NBU_GCU_EN  ;
				udelay(10) ;
			}
			mmio_bridge = cpu_io_mmio(peerid,BRIDGE) ;
			mmio_bridge[NBU_MODE_REG] = mmio_bridge[NBU_MODE_REG] | NBU_GCU_EN ;
			peerid++ ;
		} else {
			link_disable(mmio) ;
		}
	}
	return peerid ;
}


/* Called from lib/board.c
 */
int ici_init(void) 
{
	num_nodes = enumerate(0) ;
	printf("Nodes online: %d\n",num_nodes) ;
	return 0;
}
