#ifndef __DDR3_SPD_H_
#define __DDR3_SPD_H_

#define SPD_DBG
#define SPD_ERR

#ifdef  SPD_DBG
#define spd_dbg(fmt,args...)     printf(fmt,##args)
#else
#define spd_dbg(fmt,args...)
#endif

#define NUM_SPD_BYTES       	256

#define SPD_RDIMM_TYPE    		0x1
#define SPD_UDIMM_TYPE    		0x2
#define SPD_SODIMM_TYPE   		0x3
#define SPD_MICRDIMM_TYPE 		0x4
#define SPD_MINUDIMM_TYPE 		0x5
#define SPD_MINRDIMM_TYPE 		0x6

#define SPD_DEV_X4   			0
#define SPD_DEV_X8   			1
#define SPD_DEV_X16  			2
#define SPD_DEV_X32  			3

#define SPD_MOD_X8   			0
#define SPD_MOD_X16  			1
#define SPD_MOD_X32  			2
#define SPD_MOD_X64  			3

#define SPD_CRC_117(x)   		((x[0] >> 7) & 0x1)
#define SPD_REV(x)       		(x[1])
#define SPD_DDR3(x)      		(x[2] == 0xB)
#define SPD_DIMM_TYP(x)  		(x[3] & 0xf)
#define SPD_CAPACITY(x)  		(x[4] & 0x7)
#define SPD_BANK_BITS(x) 		((x[4]>>4) & 0x7)
#define SPD_COL_BITS(x)  		((x[5]>>0) & 0x7)
#define SPD_ROW_BITS(x)  		((x[5]>>3) & 0x7)
#define SPD_135V(x)      		((x[6]>>1) & 0x1)
#define SPD_DEV_WIDTH(x) 		((x[7]>>0) & 0x7)
#define SPD_NUM_RANKS(x) 		((x[7]>>3) & 0x7)
#define SPD_MOD_WIDTH(x) 		((x[8]>>0) & 0x7)
#define SPD_ECC(x)       		((x[8]>>3) & 0x1)
#define SPD_FTB(x)       		((((x[8]>>4) & 0xf) * 1000)/((x[8]>>0) & 0xf))
#define SPD_MTB(x)       		((x[10] * 1000)/x[11])
#define SPD_tCKmin(x)       	(x[12])
#define SPD_CASLAT(x)      		(((x[15] << 8) | x[14]) << 4)
#define SPD_tAAmin(x)      		(x[16])
#define SPD_tWRmin(x)      		(x[17])
#define SPD_tRCDmin(x)     		(x[18])
#define SPD_tRRDmin(x)     		(x[19])
#define SPD_tRPmin(x)      		(x[20])
#define SPD_tRASmin(x)     		(((x[21] & 0xf) << 8) | x[22])
#define SPD_tRCmin(x)      		((((x[21] >> 4) & 0xf) << 8) | x[23])
#define SPD_tRFCmin(x)     		((x[25] << 8)| x[24])
#define SPD_tWTRmin(x)     		(x[26])
#define SPD_tRTPmin(x)     		(x[27])
#define SPD_tFAW(x)        		(((x[28] & 0xf) << 8) | x[29])
#define SPD_RZQD6(x)       		(x[30] & 0x1)
#define SPD_RZQD7(x)       		((x[30]>>1) & 0x1)
#define SPD_DLLOFF(x)      		((x[30]>>7) & 0x1)
#define SPD_MIRROR(x)        	(x[63] & 0x1)
#define SPD_CRC(x)         		((x[127] << 8)| x[126])

#endif /* __DDR3_SPD_H_ */
